import datetime as dt
from enum import Enum
from typing import Any, Dict

from pydantic import BaseModel

from ..utils import dict_to_camel_case, dict_to_snake_case

REGEX_NUMERIC = r'^\d*$'
REGEX_CAP_NUM = r'^[A-Z0-9\-\s]*$'


class Resource(BaseModel):
    _date_format = '%Y%m%d'

    @classmethod
    def from_dict(cls, d: Dict[str, Any]):
        transformed = dict_to_snake_case(d)
        return cls(**transformed)

    def dict(self, to_camel_case: bool = False, *args, **kwargs):
        d = super().dict(*args, **kwargs)
        res = {}
        for key, value in d.items():
            if isinstance(value, dt.date) or isinstance(value, dt.datetime):
                res[key] = value.strftime(self._date_format)
            elif isinstance(value, Enum):
                res[key] = value.value
            else:
                res[key] = value
        if to_camel_case:
            res = dict_to_camel_case(res)
            res = res
        return res
